//
//  UIViewController+LocationAuthorization.m
//
//  Created by Johannes Luderschmidt on 26.09.14.
//
#import <objc/runtime.h>

#import "UIViewController+LocationAuthorization.h"

static void *LocationAuthorizationManagerKey;
@implementation UIViewController (LocationAuthorization)

-(void)initializeLocationAuthorization{
    CLLocationManager *locationManager = objc_getAssociatedObject(self, &LocationAuthorizationManagerKey);
    locationManager = [[CLLocationManager alloc] init];
    locationManager.delegate = self;
    objc_setAssociatedObject(self, &LocationAuthorizationManagerKey, locationManager, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

-(BOOL)checkLocationAuthorizationAlwaysStatus{
    BOOL locationAuthorized = YES;
    CLLocationManager *locationManager = objc_getAssociatedObject(self, &LocationAuthorizationManagerKey);
    if(!locationManager){
        [self initializeLocationAuthorization];
        locationManager = objc_getAssociatedObject(self, &LocationAuthorizationManagerKey);
    }
    if ([locationManager respondsToSelector:@selector(requestWhenInUseAuthorization)]) {
        if([CLLocationManager authorizationStatus] != kCLAuthorizationStatusAuthorizedAlways){
            SEL requestAlwaysAuthorizationSelector = NSSelectorFromString(@"requestAlwaysAuthorization");
//suppress warning in xcode
#       pragma clang diagnostic push
#       pragma clang diagnostic ignored "-Warc-performSelector-leaks"
            [locationManager performSelector:requestAlwaysAuthorizationSelector];
#       pragma clang diagnostic pop
            if([CLLocationManager authorizationStatus] == kCLAuthorizationStatusDenied){
                locationAuthorized = NO;
            }
        }
    }
    
    return locationAuthorized;
}

-(BOOL)checkLocationAuthorizationWhenInUseStatus{
    BOOL locationAuthorized = YES;
    CLLocationManager *locationManager = objc_getAssociatedObject(self, &LocationAuthorizationManagerKey);
    if(!locationManager){
        [self initializeLocationAuthorization];
        locationManager = objc_getAssociatedObject(self, &LocationAuthorizationManagerKey);
    }
    if ([locationManager respondsToSelector:@selector(requestWhenInUseAuthorization)]) {
        if([CLLocationManager authorizationStatus] < kCLAuthorizationStatusAuthorizedAlways){
            SEL requestWhenInUseAuthorizationSelector = NSSelectorFromString(@"requestWhenInUseAuthorization");
//suppress warning in xcode
#       pragma clang diagnostic push
#       pragma clang diagnostic ignored "-Warc-performSelector-leaks"
            [locationManager performSelector:requestWhenInUseAuthorizationSelector];
#       pragma clang diagnostic pop
            if([CLLocationManager authorizationStatus] == kCLAuthorizationStatusDenied){
                locationAuthorized = NO;
            }
        }
    }
    
    return locationAuthorized;
}

#pragma mark - CLLocationManagerDelegate Events

-(void)locationManager:(CLLocationManager *)manager didChangeAuthorizationStatus:(CLAuthorizationStatus)status{
    if( status==kCLAuthorizationStatusDenied){
        [self alertLocationDeniedError];
    }
}

-(void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error {
    NSLog(@"%@",error);
    switch ([error code]) {
        case kCLErrorLocationUnknown:
            NSLog(@"%@",error); break;
        case kCLErrorDenied:
            [self alertLocationDeniedError]; break;
        case kCLErrorNetwork:
            NSLog(@"%@",error); break;
        default:
            break;
    }
}

-(void)alertLocationDeniedError {
//    UIAlertView *alert=[[UIAlertView alloc] initWithTitle:@"Location denied" message:@"You denied accessing your location for this app. Without location updates there are many features in this app that you can't use. You can activate location updates by opening the Setting app and activating  Privacy > Location > [This app's name] 'While Using this app' or 'Always'." delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
//    [alert show];
}

@end
